@@ -1,8 +1,8 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.45.2.1 2009/03/08 14:58:15 gleu Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.45.2.5 2009/08/22 04:19:49 t-ishii Exp $
  *
- * pgpool: a language independent connection pool server for PostgreSQL 
+ * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
  * Copyright (c) 2003-2009	PgPool Global Development Group
@@ -121,7 +121,7 @@ ProcessInfo *pids;	/* shmem child pid table */
  * this is a two dimension array. i.e.:
  * con_info[pool_config->num_init_children][pool_config->max_pool]
  */
-ConnectionInfo *con_info;		
+ConnectionInfo *con_info;
 
 static int unix_fd;	/* unix domain socket fd */
 static int inet_fd;	/* inet domain socket fd */
@@ -247,7 +247,7 @@ int main(int argc, char **argv)
 					exit(1);
 				}
 				break;
-				
+
 			case 'n':	/* no detaching control ttys */
 				not_detach = 1;
 				break;
@@ -428,6 +428,14 @@ int main(int argc, char **argv)
 	}
 	*InRecovery = 0;
 
+	/*
+	 * We need to block signal here. Otherwise child might send some
+	 * signals, for example SIGUSR1(fail over).  Children will inherit
+	 * signal blocking but they do unblock signals at the very beginning
+	 * of process.  So this is harmless.
+	 */
+	POOL_SETMASK(&BlockSig);
+
 	/* fork the children */
 	for (i=0;i<pool_config->num_init_children;i++)
 	{
@@ -436,7 +444,7 @@ int main(int argc, char **argv)
 	}
 
 	/* set up signal handlers */
-	POOL_SETMASK(&BlockSig);
+
 	pool_signal(SIGTERM, exit_handler);
 	pool_signal(SIGINT, exit_handler);
 	pool_signal(SIGQUIT, exit_handler);
@@ -528,7 +536,7 @@ int main(int argc, char **argv)
 					else
 					{
 						retrycnt++;
-						pool_signal(SIGALRM, SIG_IGN);
+						pool_signal(SIGALRM, SIG_IGN);	/* Cancel timer */
 
 						if (retrycnt > NUM_BACKENDS)
 						{
@@ -771,7 +779,7 @@ static void write_pid_file(void)
 pid_t pcp_fork_a_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 {
 	pid_t pid;
-	
+
 	pid = fork();
 
 	if (pid == 0)
@@ -954,7 +962,7 @@ static void myunlink(const char* path)
 static void myexit(int code)
 {
 	int i;
-	
+
 	if (getpid() != mypid)
 		return;
 
@@ -975,7 +983,7 @@ static void myexit(int code)
 			pool_error("wait() failed. reason:%s", strerror(errno));
 		POOL_SETMASK(&UnBlockSig);
 	}
-	
+
 	myunlink(un_addr.sun_path);
 	myunlink(pcp_un_addr.sun_path);
 	myunlink(pool_config->pid_file_name);
@@ -1009,7 +1017,7 @@ void degenerate_backend_set(int *node_id_set, int count)
 		if (node_id_set[i] < 0 || node_id_set[i] >= MAX_NUM_BACKENDS ||
 			!VALID_BACKEND(node_id_set[i]))
 		{
-			pool_log("notice_backend_error: node %d is not valid backend.");
+			pool_log("notice_backend_error: node %d is not valid backend.", i);
 			continue;
 		}
 
@@ -1031,7 +1039,7 @@ void send_failback_request(int node_id)
 
 	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS || VALID_BACKEND(node_id))
 	{
-		pool_error("send_failback_request: node %d is alive.");
+		pool_error("send_failback_request: node %d is alive.", node_id);
 		return;
 	}
 
@@ -1187,7 +1195,7 @@ static void failover(void)
 		return;
 	}
 
-	/* 
+	/*
 	 * if not in replication mode/master slave mode, we treat this a restart request.
 	 * otherwise we need to check if we have already failovered.
 	 */
@@ -1380,6 +1388,7 @@ static RETSIGTYPE health_check_timer_handler(int sig)
 int health_check(void)
 {
 	int fd;
+	int sts;
 
 	/* V2 startup packet */
 	typedef struct {
@@ -1435,7 +1444,28 @@ int health_check(void)
 			return i+1;
 		}
 
-		read(fd, &kind, 1);
+		/*
+		 * Don't bother to be blocked by read(2). It will be
+		 * interrupted by ALRAM anyway.
+		 */
+		sts = read(fd, &kind, 1);
+		if (sts == -1)
+		{
+			pool_error("health check failed during read. host %s at port %d is down. reason: %s",
+					   BACKEND_INFO(i).backend_hostname,
+					   BACKEND_INFO(i).backend_port,
+					   strerror(errno));
+			close(fd);
+			return i+1;
+		}
+		else if (sts == 0)
+		{
+			pool_error("health check failed. EOF encountered. host %s at port %d is down",
+					   BACKEND_INFO(i).backend_hostname,
+					   BACKEND_INFO(i).backend_port);
+			close(fd);
+			return i+1;
+		}
 
 		/*
 		 * If a backend raised a FATAL error(max connections error or
@@ -1488,7 +1518,7 @@ system_db_health_check(void)
 	/* if SystemDB is already down, ignore */
 	if (SYSDB_STATUS == CON_UNUSED || SYSDB_STATUS == CON_DOWN)
 		return 0;
-	
+
 	if (*SYSDB_INFO->hostname == '\0')
 		fd = connect_unix_domain_socket_by_port(SYSDB_INFO->port, pool_config->backend_socket_dir);
 	else
@@ -1583,7 +1613,7 @@ static void reaper(void)
 			break;
 		} else {
 			pool_debug("child %d exits with status %d by signal %d", pid, status, WTERMSIG(status));
-		
+
 			/* look for exiting child's pid */
 			for (i=0;i<pool_config->num_init_children;i++)
 			{
@@ -1665,12 +1695,12 @@ pool_get_system_db_info(void)
 {
 	if (system_db_info == NULL)
 		return NULL;
-	
+
 	return system_db_info->info;
 }
 
 
-/* 
+/*
  * handle SIGUSR2
  * Wakeup all processes
  */
@@ -1682,10 +1712,10 @@ static void wakeup_children(void)
 
 static RETSIGTYPE wakeup_handler(int sig)
 {
-	POOL_SETMASK(&BlockSig);	
+	POOL_SETMASK(&BlockSig);
 	wakeup_request = 1;
 	write(pipe_fds[1], "\0", 1);
-	POOL_SETMASK(&UnBlockSig);	
+	POOL_SETMASK(&UnBlockSig);
 }
 
 /*
@@ -1785,7 +1815,7 @@ static void pool_sleep(unsigned int second)
 
 		r = pool_pause(&timeout);
 		POOL_SETMASK(&BlockSig);
-		if (r > 0)									
+		if (r > 0)
 			CHECK_REQUEST;
 		POOL_SETMASK(&UnBlockSig);
 		gettimeofday(&current_time, NULL);
@@ -1840,7 +1870,7 @@ static int trigger_failover_command(int node, const char *command_line)
 		return -1;
 	}
 	exec_cmd = init_string("");
-	
+
 	while (*command_line)
 	{
 		if (*command_line == '%')
