@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pcp_child.c,v 1.7 2007/10/12 04:35:31 y-asaba Exp $
- * 
- * pgpool: a language independent connection pool server for PostgreSQL 
+ * $Header: /cvsroot/pgpool/pgpool-II/pcp_child.c,v 1.11.2.1 2009/08/22 04:19:49 t-ishii Exp $
+ *
+ * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2007	PgPool Global Development Group
+ * Copyright (c) 2003-2008	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -66,10 +66,14 @@ static void unset_nonblock(int fd);
 static int user_authenticate(char *buf, char *passwd_file, char *salt, int salt_len);
 static void pool_random_salt(char *md5Salt);
 static RETSIGTYPE wakeup_handler(int sig);
+static RETSIGTYPE reload_config_handler(int sig);
 
 extern int myargc;
 extern char **myargv;
 
+static volatile sig_atomic_t pcp_got_sighup = 0;
+volatile sig_atomic_t pcp_wakeup_request = 0;
+
 void
 pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 {
@@ -97,14 +101,14 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 	/* set up signal handlers */
 	signal(SIGTERM, die);
 	signal(SIGINT, die);
-	signal(SIGHUP, die);
+	signal(SIGHUP, reload_config_handler);
 	signal(SIGQUIT, die);
 	signal(SIGCHLD, SIG_DFL);
 	signal(SIGUSR1, SIG_DFL);
 	signal(SIGUSR2, wakeup_handler);
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGALRM, SIG_IGN);
-	
+
 	for(;;)
 	{
 		errno = 0;
@@ -113,10 +117,8 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 		{
 			frontend = pcp_do_accept(unix_fd, inet_fd);
 			if (frontend == NULL)
-			{
-				/* pcp_do_accept() calls pool_error(), so just exit */
-				exit(1);
-			}
+				continue;
+
 			unset_nonblock(frontend->fd);
 		}
 
@@ -175,7 +177,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 		/* is this connection authenticated? if not disconnect immediately*/
 		if ((! authenticated) && (tos != 'R' && tos != 'M'))
 		{
-			pool_debug("pcp_child: connection not authorized");					
+			pool_debug("pcp_child: connection not authorized");
 			free(buf);
 			buf = NULL;
 			pcp_close(frontend);
@@ -188,7 +190,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 		pool_debug("pcp_child: received PCP packet type of service '%c'", tos);
 
 		set_ps_display("PCP: processing a request", false);
-			
+
 		switch (tos)
 		{
 			case 'R':			/* authentication */
@@ -197,7 +199,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 
 				if (random_salt)
 				{
-					authenticated = user_authenticate(buf, pcp_conf_file, salt, 4);					
+					authenticated = user_authenticate(buf, pcp_conf_file, salt, 4);
 				}
 				if (!random_salt || !authenticated)
 				{
@@ -268,7 +270,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 				char mesg[16];
 
 				snprintf(mesg, sizeof(mesg), "%d", node_count);
-				
+
 				pcp_write(frontend, "l", 1);
 				wsize = htonl(sizeof(code) +
 							  strlen(mesg)+1 +
@@ -289,7 +291,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 			{
 				int node_id;
 				int wsize;
-				
+
 				BackendInfo *bi = NULL;
 
 				node_id = atoi(buf);
@@ -320,7 +322,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 					snprintf(port_str, sizeof(port_str), "%d", bi->backend_port);
 					snprintf(status, sizeof(status), "%d", bi->backend_status);
 					snprintf(weight_str, sizeof(weight_str), "%f", bi->backend_weight);
-				
+
 					pcp_write(frontend, "i", 1);
 					wsize = htonl(sizeof(code) +
 								  strlen(bi->backend_hostname)+1 +
@@ -374,7 +376,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 					snprintf(mesg+total_port_len, strlen(port)+1, "%s", port);
 					total_port_len += strlen(port)+1;
 				}
-				
+
 				pcp_write(frontend, "n", 1);
 				wsize = htonl(sizeof(code) +
 							  strlen(process_count_str)+1 +
@@ -389,19 +391,19 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 					pool_error("pcp_child: pcp_flush() failed. reason: %s", strerror(errno));
 					exit(1);
 				}
-				
+
 				free(process_list);
 				free(mesg);
 
 				pool_debug("pcp_child: %d process(es) found", process_count);
 				break;
 			}
-				
+
 			case 'P':			/* process info */
 			{
 				int proc_id;
 				int wsize;
-				
+
 				ProcessInfo *pi = NULL;
 
 				proc_id = atoi(buf);
@@ -508,7 +510,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 			case 'S':			/* SystemDB info */
 			{
 				int wsize;
-				
+
 				SystemDBInfo *si = NULL;
 				si = pool_get_system_db_info();
 
@@ -583,7 +585,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 						int type_list_offset;
 						DistDefInfo *ddi;
 						int i, j;
-					
+
 						for (i = 0; i < si->dist_def_num; i++)
 						{
 							ddi = &si->dist_def_slot[i];
@@ -603,7 +605,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 								pool_error("pcp_child: malloc() failed. reason: %s", strerror(errno));
 								exit(1);
 							}
-						
+
 							col_list_offset = type_list_offset = 0;
 							for (j = 0; j < ddi->col_num; j++)
 							{
@@ -662,13 +664,13 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 				}
 				break;
 			}
-			
+
 			case 'D':			/* detach node */
 			{
 				int node_id;
 				int wsize;
 				char code[] = "CommandComplete";
-				
+
 				node_id = atoi(buf);
 				pool_debug("pcp_child: detaching Node ID %d", node_id);
 				notice_backend_error(node_id);
@@ -690,7 +692,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 				int node_id;
 				int wsize;
 				char code[] = "CommandComplete";
-				
+
 				node_id = atoi(buf);
 				pool_debug("pcp_child: attaching Node ID %d", node_id);
 				send_failback_request(node_id);
@@ -706,7 +708,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 				}
 				break;
 			}
-			
+
 			case 'T':
 			{
 				char mode = buf[0];
@@ -752,7 +754,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 				}
 				else
 				{
-					pool_debug("pcp_child: start online recovery");				
+					pool_debug("pcp_child: start online recovery");
 					node_id = atoi(buf);
 
 					r = start_recovery(node_id);
@@ -785,7 +787,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 			case 'F':
 				pool_debug("pcp_child: stop online recovery");
 				break;
-			
+
 			case 'X':			/* disconnect */
 				pool_debug("pcp_child: client disconnecting. close connection");
 				authenticated = 0;
@@ -834,7 +836,7 @@ die(int sig)
 static RETSIGTYPE
 wakeup_handler(int sig)
 {
-
+	pcp_wakeup_request = 1;
 }
 
 static PCP_CONNECTION *
@@ -864,7 +866,7 @@ pcp_do_accept(int unix_fd, int inet_fd)
 	{
 		if (errno == EAGAIN || errno == EINTR)
 			return NULL;
-		
+
 		pool_error("pcp_child: select() failed. reason: %s", strerror(errno));
 		return NULL;
 	}
@@ -894,6 +896,12 @@ pcp_do_accept(int unix_fd, int inet_fd)
 		return NULL;
 	}
 
+	if (pcp_got_sighup)
+	{
+		pool_get_config(get_config_file_name(), RELOAD_CONFIG);
+		pcp_got_sighup = 0;
+	}
+
 	pool_debug("I am PCP %d accept fd %d", getpid(), afd);
 
 	if (inet)
@@ -934,7 +942,7 @@ unset_nonblock(int fd)
 {
 	int var;
 
-	/* set fd to none blocking */
+	/* set fd to non-blocking */
 	var = fcntl(fd, F_GETFL, 0);
 	if (var == -1)
 	{
@@ -966,7 +974,7 @@ user_authenticate(char *buf, char *passwd_file, char *salt, int salt_len)
 
 	/* strcpy() should be OK, but use strncpy() to be extra careful */
 	strncpy(packet_username, buf, MAX_USER_PASSWD_LEN);
-	index = memchr(buf, '\0', MAX_USER_PASSWD_LEN) + 1;
+	index = (char *) memchr(buf, '\0', MAX_USER_PASSWD_LEN) + 1;
 	if (index == NULL)
 	{
 		pool_debug("pcp_child: error while reading authentication packet");
@@ -1058,3 +1066,9 @@ static void pool_random_salt(char *md5Salt)
 	rand = random();
 	md5Salt[3] = (rand % 255) + 1;
 }
+
+/* SIGHUP handler */
+static RETSIGTYPE reload_config_handler(int sig)
+{
+	pcp_got_sighup = 1;
+}
