@@ -1,7 +1,7 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.36 2009/01/25 10:13:15 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.36.2.4 2009/07/22 08:46:55 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -78,6 +78,9 @@ typedef enum {
 #define PROTO_MAJOR_V2	2
 #define PROTO_MAJOR_V3	3
 
+/* Cancel packet proto major */
+#define PROTO_CANCEL	80877102
+
 /*
  * In protocol 3.0 and later, the startup packet length is not fixed, but
  * we set an arbitrary limit on it anyway.	This is just to prevent simple
@@ -431,9 +434,18 @@ extern char query_string_buffer[];		/* last query string sent to simpleQuery() *
  */
 extern char *get_config_file_name(void);
 extern char *get_hba_file_name(void);
+#ifdef __GNUC__
+extern void pool_error(const char *fmt,...)
+   	__attribute__((format (printf, 1, 2)));
+extern void pool_debug(const char *fmt,...)
+   	__attribute__((format (printf, 1, 2)));
+extern void pool_log(const char *fmt,...)
+   	__attribute__((format (printf, 1, 2)));
+#else
 extern void pool_error(const char *fmt,...);
 extern void pool_debug(const char *fmt,...);
 extern void pool_log(const char *fmt,...);
+#endif
 extern int pool_init_config(void);
 extern int pool_get_config(char *confpath, POOL_CONFIG_CONTEXT context);
 extern void do_child(int unix_fd, int inet_fd);
@@ -483,6 +495,7 @@ extern int connect_unix_domain_socket(int secondary_backend);
 extern int connect_inet_domain_socket_by_port(char *host, int port);
 extern int connect_unix_domain_socket_by_port(int port, char *socket_dir);
 
+extern void pool_set_timeout(int timeoutval);
 extern int pool_check_fd(POOL_CONNECTION *cp);
 
 extern void pool_send_frontend_exits(POOL_CONNECTION_POOL *backend);
@@ -582,5 +595,6 @@ extern void finish_recovery(void);
 /* child.c */
 extern void pool_set_nonblock(int fd);
 extern void pool_unset_nonblock(int fd);
+extern void cancel_request(CancelPacket *sp);
 
 #endif /* POOL_H */
